// -*- C++ -*-
// $RCSfile: threadstate.h,v $
// $Revision: 1.4 $
// $Author: vrc $
// $Date: 2007-09-26 22:02:16 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

//  Objects and functions for storing and retrieving thread
//  information.  There is exactly one ThreadState object per thread.
//  There can be more than one ThreadID object for one thread, but
//  they are all equal.  The ThreadID is used by findThreadState to
//  retrieve the thread's ThreadState.

#ifndef THREADSTATE
#define THREADSTATE

#include <Python.h>
#include <pthread.h>

void releasePyLock();
void acquirePyLock();

int findThreadNumber();

void disable_threading();

class ThreadID {
private:
  pthread_t _ID;
public:
  ThreadID();
  const pthread_t & get_ID() const { return _ID; }
};

int operator==(ThreadID&, ThreadID&);




class ThreadState {
 private:
  //Python interpreter lock variables
  PyThreadState *_state;
  ThreadID _ID;	// may be re-used? depends on pthread implementation?

  // counter tells how many times the Python Interpreter lock has been
  // acquired by the current thread. When ThreadState is constructed
  // its default value is one because the Python acquires the lock
  // automatically. acquirePyLock and releasePyLock manipulate the
  // counter.
  int counter;
  int _id;			// unique among all threads for all time


  bool defunct; //flag for removing unused ThreadState objects from static list
public:
  ThreadState();
  ~ThreadState();
  void acquirePyLock(); 
  void releasePyLock();
  int id() const { return _id; }
  ThreadID & get_thread_ID();
  
  bool get_defunct() const {return defunct;};
  inline void make_defunct() {defunct = true;};
  void remove_from_list();
  
  
};

void make_thread_main(); // Make the calling thread the new "main" thread.

int operator==(ThreadState&, ThreadState&);
ThreadState *findThreadState();
void cleanList();

void cancelThread(ThreadState &tobecancelled);
void testcancel();

#endif // THREADSTATE
