// -*- C++ -*-
// $RCSfile: log_value.h,v $
// $Revision: 1.5 $
// $Author: vrc $
// $Date: 2007-09-26 22:02:44 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include <math.h>
/* 
   LogValue is a class designed to transform the current relative residual of the IML++
   solvers into a number that is a measure of the closeness to convergence of the
   utilized method.
*/

#ifndef LOGVALUE
#define LOGVALUE


class LogValue {
private:
  double current_value;
  double tolerance;
  double initial_residual;
  double log_res_over_tol;
  double logvalue(const double) const;

public:
  
  LogValue(const double ro, const double tol)
    : tolerance(tol),
      initial_residual(ro)
  {
    log_res_over_tol = log(initial_residual/tolerance);
  }
  LogValue()
    : tolerance(0.1),
      initial_residual(1.0)
  { 
    log_res_over_tol = log(initial_residual/tolerance); 
  }
  double operator() (const double val) { return logvalue(val); };
};




#endif // LOGVALUE

