# -*- python -*-
# $RCSfile: materialplugin.py,v $
# $Revision: 1.9 $
# $Author: rlua $
# $Date: 2007-09-28 03:20:47 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

## Define a MicrostructurePlugIn to catch "material changed"
## switchboard signals and tell the relevant subproblems of the event.
from ooflib.SWIG.common import switchboard
from ooflib.SWIG.engine import material
from ooflib.common import debug
from ooflib.common import microstructure
from ooflib.engine import materialmanager
from ooflib.engine import subproblemcontext
from ooflib.engine import conjugate


class MaterialMSPlugIn(microstructure.MicrostructurePlugIn):
    def __init__(self, ms):
        microstructure.MicrostructurePlugIn.__init__(self, ms)
        self.sbcallback = switchboard.requestCallback("material changed",
                                                      self.matChanged)
    def destroy(self):
        switchboard.removeCallback(self.sbcallback)
        microstructure.MicrostructurePlugIn.destroy(self)
    def matChanged(self, materialname):
        # The given material has changed.  Look through the materials
        # in our Microstructure to see if we care.
        matl = materialmanager.getMaterial(materialname)
        #Interface branch
        #TODO: Should we look at the material associated with the edgements instead?
        interfacemsplugin=self.microstructure.getPlugIn("Interfaces")
        interfacemats=interfacemsplugin.getInterfaceMaterials()
        if matl in material.getMaterials(self.microstructure) or \
               materialname in interfacemats:
            # We *do* care. Really.
            # Tell all the SubProblems belonging to the microstructure that
            # the material has changed.
            msname = self.microstructure.name()
            subppaths = subproblemcontext.subproblems.keys(base=msname)
            for subppath in subppaths:
                subproblemcontext.subproblems[[msname]+subppath].changed()
                conjugate.check_symmetry([msname]+subppath)
                

microstructure.registerMicrostructurePlugIn(MaterialMSPlugIn, "Material")
