# -*- python -*-
# $RCSfile: odedriver.py,v $
# $Revision: 1.3 $
# $Author: langer $
# $Date: 2007-10-05 14:56:59 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from ooflib.common import registeredclass

class ODEDriver(registeredclass.RegisteredClass):
    registry = []
    def initialize(self, mesh, subproblems, time):
        for subproblem in self.subproblems:
            subproblem.initialize(time)
        mesh.time = time        # ??

## ODEDriver should be a RegisteredClass, so the __init__s should only
## set parameters.  They shouldn't actually integrate.

class AdaptiveDriver(ODEDriver):
    def __init__(self, mesh, subproblems, timesteppers, outputschedule,
                 tolerance):
        time = outputschedule.start()
        self.initialize(mesh, subproblems, outputschedule)
        outputschedule.run_outputs(time)
        targettime = t1
        nsubs = len(subproblems)
        for t1 in outputschedule.times():
            self.enddata = mesh.copyDoFs() # C++ obj containing all dof values
            while time < t1:               # WHAT ABOUT STATIC?
                # Advance up to t1 in self-consistent steps.  A
                # consistent step is one in which all timesteppers get
                # to the same point, independent of the order in which
                # they proceed.
                consistent = False
                self.times = [None]*nsubs # times attained by each subproblem
                deltas = [None]*nsubs     # recommended next step size
                prevtry = None  # dof values after last attempted step
                while not consistent:
                    newconstraints = False # constraint eqs added or removed?
                    success = True # did all subps get to targettime?
                    for which in range(len(subproblems)):
                        subproblem = subproblems[which]
                        timestepper = timesteppers[which]
                        # qcStep doesn't return until it has taken a
                        # step that satisfies the tolerance.  The
                        # return values are the actual time attained,
                        # and a recommendation for the next step
                        # delta.  delta may be None if no
                        # recommendation can be made.
                        achieved, delta = timestepper.qcStep(self,
                                                             subproblem,
                                                             self.enddata,
                                                             time, targettime,
                                                             tolerance)
                        self.times[which] = achieved
                        success = success and achieved==targettime
                        deltas[which] = delta
                        # Ask subproblems if any constraints have
                        # changed.  New equations and dofs may have
                        # been added, or Property values changed.
                        constrnt = subproblem.applyConstraints(self.enddata,
                                                               achieved)
                        newconstraints = newconstraints or constrnt
                    # end loop over subproblems
                    consistent = ((not newconstraints) and success and
                                  not (nsubs > 1 and
                                       (prevtry is None or
                                        self.enddata-prevtry > tolerance)))
                    if not consistent:
                        prevtry = self.enddata.copy()
                        targettime = min(self.times)
                # end "while not consistent".  All subproblems have
                # evolved to targettime.
                time = targettime
                for subproblem in subproblems:
                    subproblem.discard_matrices_predating(time)
                mesh.updateDoFs(self.enddata, time) # ?
                okdeltas = [d for d in deltas if d is not None]
                if okdeltas:
                    targettime = time + min(okdeltas)
                    if targettime > t1:
                        targettime = t1
                else:   # no recommended next step available
                    targettime = t1
            # end "while time < t1"
            assert(time == t1)
            outputschedule.run_outputs(time) 
        
    def getDoF(self, index, time):
        # This needs to be in C++, but this is something like what
        # it would sort of look like if it were in Python. It
        # needs to be in C++ so that Properties can use it.
        t0 = self.mesh.time
        t1 = self.enddata[index].time
        v0 = self.mesh.dof[index].value
        if t1 == t0:
            return v0
        v1 = self.enddata[index].value
        return v0 + (v1 - v0)*(time - t0)/(t1 - t0)




class StaticDriver(ODEDriver):
    def __init__(self, mesh, subproblems, matrixmethods, time, tolerance):
        self.initialize(mesh, subproblems, time)
        consistent = False
        prevtry = None
        while not consistent:
            newconstraints = False
            for which in range(len(subproblems)):
                subproblem = subproblems[which]
                matrixmethod = matrixmethods[which]
                # timestepper must be a StaticQCStepper
                self.enddata = mesh.copyDoFs()
                self.solve(self, subproblem, matrixmethod, tolerance,
                           self.enddata)
                constrnt = subproblem.applyConstraints(self.enddata, time)
                newconstraints = newconstraints or constrnt
            consistent = ((not newconstraints) and (prevtry is not None)
                          and (self.enddata-prevtry < tolerance))
            if not consistent:
                prevtry = self.enddata
        ## end "while not consistent"
        mesh.updateDoFs(self.enddata, time)
